@@ -4,23 +4,59 @@ import plotly.graph_objects as go
 import pandas as pd
 import numpy as np
 
-from insider.mixins.price import PriceIndicatorMixin
-from insider.mixins.volume import VolumnIndicatorMixin
+from insider.indicators.price import PriceIndicatorMixin
+from insider.indicators.volume import VolumnIndicatorMixin
+from insider.indicators.sar import SARIndicatorMixin
 from insider.stock import Stock
-from insider.constants import MA_N, MD_N, EXPMA_N, RSI_N, MIKE_COLS
+from insider.utils import set_layout
+from insider.constants import (
+    MA_N,
+    MD_N,
+    EXPMA_N,
+    RSI_N,
+    MIKE_COLS,
+    CDP_COLS,
+    EXTERNAL_COLS,
+)
 
 
-class StockInsider(
-    Stock, PriceIndicatorMixin, VolumnIndicatorMixin,
-):
+class StockInsider(Stock, PriceIndicatorMixin, VolumnIndicatorMixin, SARIndicatorMixin):
     """Plot daily trading indicators."""
 
-    def __init__(self, code, ktype="D"):
+    def __init__(self, code, ktype="D", df=None):
         """
-        code: Full stock code，(e.g. 'sz002156')，股票完整代码
-        ktype: Data frequency, valid input is `D`, `W`, or `M`. 股票数据的频率
+        Parameters:
+            code: Full stock code，(e.g. 'sz002156')，股票完整代码
+            ktype: Data frequency, valid input is `D`, `W`, or `M`. 股票数据的频率
         """
-        super().__init__(code, ktype)
+        if df is not None and isinstance(df, pd.DataFrame):
+            self._df = df
+            self.stock_code = code
+        else:
+            super().__init__(code, ktype)
+
+    @classmethod
+    def from_external_csv_data(cls, fpath: str, code=None):
+        """Allow the StockInsider class serve for external stock data (other than
+        default Chinese stock data) to visualize stock trading indicators.
+        如果你有已经下载好的或者是非中国股市股票的数据想要来计算和绘出交易指标，你可以选择
+        用这个classmethod来初始化。
+
+        Parameters:
+            fpath: the path to the external stock data in CSV. 外部数据的路径
+            code: the stock code you would like to show in each visualization, the default
+                is None which will have `external data` shown as stock name.
+                股票代码，这个没有必要是真实的代码，你选择的结果将会被以股票代码的形式展示。
+        """
+        df = pd.read_csv(fpath)
+        if not set(EXTERNAL_COLS).issubset(df.columns):
+            raise ValueError(
+                f"{EXTERNAL_COLS} are mandatory in external data so as to plot"
+                f" all trading indicators."
+            )
+        if code is None:
+            code = "external data"
+        return cls(code=code, df=df)
 
     @staticmethod
     def _plot_line(df: pd.DataFrame, head: int, line_name: str, y: str = "close"):
@@ -51,13 +87,34 @@ class StockInsider(
             verbose_data = verbose_func(df, head)
             plot_data.append(verbose_data)
 
-        layout = self._set_layout()
+        layout = set_layout()
         fig = go.Figure(data=plot_data, layout=layout)
         if verbose:
             fig.update_layout(xaxis_rangeslider_visible=False)
         fig.update_layout(title_text=f"{name.upper()} Chart ({self.stock_code})")
         fig.show()
 
+    def _plot(self, df, head, title, lines, verbose: bool = False):
+        """General plot functions shared across the class."""
+        fig = go.Figure(layout=set_layout())
+
+        if isinstance(lines, str):
+            lines = [lines]
+        elif not isinstance(lines, list):
+            raise ValueError("Only string or list is valid type for lines.")
+
+        for n in lines:
+            fig.add_trace(self._plot_line(df, head=head, y=n, line_name=n.upper()))
+
+        if verbose:
+            fig.add_trace(self._plot_stock_data(self._df, head))
+
+        fig.update_layout(
+            title_text=f"{title} Chart ({self.stock_code})",
+            xaxis_rangeslider_visible=False,
+        )
+        fig.show()
+
     def plot_ma(
         self, head: int = 90, ns: Optional[List[int]] = None, verbose: bool = False
     ):
@@ -140,7 +197,7 @@ class StockInsider(
         )
 
     def plot_macd(self, head: int = 90):
-        """Plot MACD Indicator. 绘出MACD曲线
+        """Plot MACD (Moving Average Convergence and Divergence) Indicator. 绘出MACD曲线
 
         Parameters:
             head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
@@ -158,7 +215,7 @@ class StockInsider(
             lambda x: "red" if x >= 0 else "green"
         )
 
-        layout = self._set_layout()
+        layout = set_layout()
         fig = go.Figure(layout=layout)
 
         fig.add_trace(
@@ -202,13 +259,13 @@ class StockInsider(
         for col in ["K", "D", "J"]:
             plot_data.append(self._plot_line(df_kdj, head, col, y=col))
 
-        layout = self._set_layout()
+        layout = set_layout()
         fig = go.Figure(data=plot_data, layout=layout)
         fig.update_layout(title_text=f"KDJ Chart ({self.stock_code})")
         fig.show()
 
     def plot_rsi(self, head: int = 90, ns: Optional[List] = None):
-        """Plot RSI Indicator. 绘出RSI曲线。
+        """Plot RSI (Relative Strength Index) Indicator. 绘出RSI曲线。
 
         Parameters:
             head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
@@ -232,7 +289,7 @@ class StockInsider(
         )
 
     def plot_vrsi(self, head: int = 90, ns: Optional[List] = None):
-        """Plot VRSI Indicator. 绘出VRSI曲线。
+        """Plot VRSI (Volumn Relative Strength Index) Indicator. 绘出VRSI曲线。
 
         Parameters:
             head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
@@ -284,7 +341,7 @@ class StockInsider(
         df_volumn = self._df.copy()
         data = self._plot_volumn_data(df_volumn, head)
 
-        layout = self._set_layout()
+        layout = set_layout()
         fig = go.Figure(data=[data], layout=layout)
         fig.update_layout(title_text=f"Volumn Chart ({self.stock_code})")
         fig.show()
@@ -318,7 +375,8 @@ class StockInsider(
         )
 
     def plot_vmacd(self, head: int = 90):
-        """Plot VMACD Indicator. 绘出VMACD曲线
+        """Plot VMACD (Volumn Moving Average Convergence and Divergence)
+        Indicator. 绘出VMACD曲线
 
         Parameters:
             head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
@@ -336,7 +394,7 @@ class StockInsider(
             lambda x: "red" if x >= 0 else "green"
         )
 
-        layout = self._set_layout()
+        layout = set_layout()
         fig = go.Figure(layout=layout)
 
         fig.add_trace(
@@ -398,19 +456,9 @@ class StockInsider(
         """
 
         df_env = self.env(n=n)
-
-        layout = self._set_layout()
-        fig = go.Figure(layout=layout)
-
-        fig.add_trace(self._plot_line(df_env, head=head, y="up", line_name="UP"))
-        fig.add_trace(self._plot_line(df_env, head=head, y="down", line_name="DOWN"))
-
-        if verbose:
-            fig.add_trace(self._plot_stock_data(self._df, head))
-        fig.update_layout(
-            title_text=f"ENV Chart ({self.stock_code})", xaxis_rangeslider_visible=False
+        self._plot(
+            df=df_env, head=head, title="ENV", lines=["up", "down"], verbose=verbose
         )
-        fig.show()
 
     def plot_vosc(self, head: int = 90):
         """Plot Volumn Oscillator Indicator 绘出成交量震荡指标
@@ -420,14 +468,7 @@ class StockInsider(
             默认90，将会绘出最近90个交易日的曲线。
         """
         df_vosc = self.vosc()
-
-        layout = self._set_layout()
-        fig = go.Figure(
-            layout=layout,
-            data=self._plot_line(df_vosc, head=head, y="vosc", line_name="VOSC"),
-        )
-        fig.update_layout(title_text=f"VOSC Chart ({self.stock_code})")
-        fig.show()
+        self._plot(df=df_vosc, head=head, title="VOSC", lines=["vosc"])
 
     def plot_mi(self, head: int = 90, n: int = 12):
         """Plot Momentum Indicator. 绘出动量指标
@@ -439,14 +480,7 @@ class StockInsider(
             是12个交易日。
         """
         df_mi = self.mi(n=n)
-
-        layout = self._set_layout()
-        fig = go.Figure(
-            layout=layout,
-            data=self._plot_line(df_mi, head=head, y="mi", line_name="MI"),
-        )
-        fig.update_layout(title_text=f"MI Chart ({self.stock_code})")
-        fig.show()
+        self._plot(df=df_mi, head=head, title="MI", lines=["mi"])
 
     def plot_mike(
         self,
@@ -455,7 +489,7 @@ class StockInsider(
         ns: Optional[List] = None,
         verbose: bool = False,
     ):
-        """Plot Mike Indicator. 绘出Mike指标
+        """Plot Mike Base Indicator. 绘出Mike指标
 
         压力线解释：
         初级压力线（WR)
@@ -477,6 +511,8 @@ class StockInsider(
         """
         if ns is None:
             ns = MIKE_COLS
+        elif isinstance(ns, str):
+            ns = [ns]
         else:
             ns = [n.lower() for n in ns]
             if [n for n in ns if n not in MIKE_COLS]:
@@ -485,18 +521,190 @@ class StockInsider(
                 )
 
         df_mike = self.mike(n=n)
+        self._plot(df=df_mike, head=head, title="MIKE", lines=ns, verbose=verbose)
 
-        layout = self._set_layout()
-        fig = go.Figure(layout=layout)
+    def plot_adtm(self, head: int = 90):
+        """Plot ADTM(23,8) indicator. 绘出动态买卖气指标 (ADTM(23, 8))
 
-        for n in ns:
-            fig.add_trace(self._plot_line(df_mike, head=head, y=n, line_name=n.upper()))
+        Parameters:
+            head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
+            默认90，将会绘出最近90个交易日的曲线。
+        """
+        df_adtm = self.adtm()
+        self._plot(df=df_adtm, head=head, title="ADTM", lines=["adtm", "adtmma"])
+
+    def plot_obv(self, head: int = 90):
+        """Plot OBV (On Balance Volumn) Indicator。绘出能量指标
+
+        Parameters:
+            head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
+            默认90，将会绘出最近90个交易日的曲线。
+        """
+        df_obv = self.obv()
+        self._plot(df=df_obv, head=head, title="OBV", lines=["obv"])
+
+    def plot_rc(self, head: int = 90, n: int = 30):
+        """Plot RC (Price rate of Change) Indicator 绘出价格变化率指标
+
+        Parameters:
+            head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
+            默认90，将会绘出最近90个交易日的曲线。
+            n: The size of moving average period for K, default is 30. 平移平均曲线的窗口大小，默认
+            是30个交易日。
+        """
+        df_rc = self.rc(n=n)
+        self._plot(df=df_rc, head=head, title="RC", lines=["arc"])
+
+    def plot_boll(self, head: int = 90, n: int = 26, verbose: bool = False):
+        """Plot BOLL line indicator 绘出布林线。
+
+        Parameters:
+            head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
+            默认90，将会绘出最近90个交易日的曲线。
+            n: The size of moving average period for K, default is 26. 平移平均曲线的窗口大小，默认
+            是26个交易日。
+            verbose: If to include stock price or not, default is False.
+            选择是否将股票价格曲线一起绘出，默认是False，将会只绘出指标曲线。
+        """
+        df_boll = self.boll(n=n)
+        self._plot(
+            df=df_boll,
+            head=head,
+            title="BOLL",
+            lines=["up", "middle", "down"],
+            verbose=verbose,
+        )
+
+    def plot_bbiboll(
+        self, head: int = 90, n: int = 11, m: int = 6, verbose: bool = False
+    ):
+        """Plot BOLL line indicator 绘出布林线。
+
+        Parameters:
+            head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
+            默认90，将会绘出最近90个交易日的曲线。
+            n: The size of moving average period for K, default is 6. 平移平均曲线的窗口大小，默认
+            是6个交易日。
+            m: The number to decide the width of up/down lines. 上下压力线的带宽倍数。
+            verbose: If to include stock price or not, default is False.
+            选择是否将股票价格曲线一起绘出，默认是False，将会只绘出指标曲线。
+        """
+        df_boll = self.bbiboll(n=n, m=m)
+        self._plot(
+            df=df_boll,
+            head=head,
+            title="BBIBOLL",
+            lines=["upr", "bbiboll", "dwn"],
+            verbose=verbose,
+        )
+
+    def plot_atr(self, head: int = 90, n: int = 14):
+        """Plot Average True Ranger indicator. 绘出真实变化率曲线
+
+        Parameters:
+            head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
+            默认90，将会绘出最近90个交易日的曲线。
+            n: The size of moving average period for K, default is 14. 平移平均曲线的窗口大小，默认
+            是14个交易日。
+        """
+        df_atr = self.atr(n=n)
+        self._plot(df=df_atr, head=head, title="ATR", lines=["tr", "atr"])
+
+    def plot_cdp(
+        self,
+        head: int = 90,
+        n: int = 1,
+        ns: Optional[List] = None,
+        verbose: bool = False,
+    ):
+        """Plot Contrarian Operation Indicator. 绘出逆势操作曲线
+
+        压力线解释：
+        CDP: 需求值
+        AH: 最高值
+        NH：近高值
+        AL：最低值
+        NL：近低值
+
+        Parameters:
+            head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
+            默认90，将会绘出最近90个交易日的曲线。
+            n: The size of moving average period for K, default is 1. 平移平均曲线的窗口大小，默认
+            是1个交易日。
+            ns: Choose the lines to be plotted, default is None, which will plot all five lines.
+            选择压力线来绘出，默认会绘出所有五条条压力线。
+            verbose: If to include stock price or not, default is False.
+            选择是否将股票价格曲线一起绘出，默认是False，将会只绘出指标曲线。
+        """
+        if ns is None:
+            ns = CDP_COLS
+        elif isinstance(ns, str):
+            ns = [ns]
+        else:
+            ns = [n.lower() for n in ns]
+            if [n for n in ns if n not in CDP_COLS]:
+                raise ValueError(
+                    "Invalid input for ns, valid values are ah, al, cdp, nh, nl."
+                )
+
+        df_cdp = self.cdp(n=n)
+        self._plot(df=df_cdp, head=head, title="CDP", lines=ns, verbose=verbose)
+
+    def plot_sar(self, head: int = 90, verbose: bool = False):
+        """Plot Stop And Reverse (SAR) indicator. 绘出止损反转指标
+
+        Parameters:
+            head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
+            默认90，将会绘出最近90个交易日的曲线。
+            verbose: If to include stock price or not, default is False.
+            选择是否将股票价格曲线一起绘出，默认是False，将会只绘出指标曲线。
+        """
+        df_sar = self.sar()
+
+        if head:
+            df_sar = df_sar.tail(head)
+
+        fig = go.Figure(layout=set_layout())
+        fig.add_trace(
+            go.Scatter(
+                x=df_sar["day"],
+                y=df_sar["sar"],
+                marker=dict(color=df_sar["color"], size=3),
+                mode="markers",
+                name="SAR",
+            )
+        )
 
         if verbose:
             fig.add_trace(self._plot_stock_data(self._df, head))
 
         fig.update_layout(
-            title_text=f"MIKE Chart ({self.stock_code})",
+            title_text=f"SAR Chart ({self.stock_code})",
             xaxis_rangeslider_visible=False,
         )
         fig.show()
+
+    def plot_mtm(self, head: int = 90):
+        """Plot MTM indicator. 绘出动量指标
+
+        Parameters:
+            head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
+            默认90，将会绘出最近90个交易日的曲线。
+        """
+        df_mtm = self.mtm()
+        self._plot(df=df_mtm, head=head, title="MTM", lines=["mtm", "mtmma"])
+
+    def plot_dmi(self, head: int = 90, n: int = 14):
+        """Plot DMI (Directional Movement Index) indicator. 绘出动向指标
+
+            Parameters:
+            head: The recent number of trading days to plot, default is 90, 最近交易日的天数，
+            默认90，将会绘出最近90个交易日的曲线。
+            n: The size of moving average period for K, default is 14. 平移平均曲线的窗口大小，默认
+            是14个交易日。
+        """
+        df_dmi = self.dmi(n=n)
+
+        self._plot(
+            df=df_dmi, head=head, title="DMI", lines=["pdi", "mdi", "adx", "adxr"]
+        )
